<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 *
 * This is the query engine.
 */

/**
 * Security - Gets user detail from database.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class USER_selectUserQuery extends PHPDS_query
{
	protected $sql = "
		SELECT
			t1.user_id, t1.user_display_name, t1.user_password, t1.user_name, t1.user_email, t1.user_group, t1.user_role, t1.language, t1.timezone as user_timezone, t1.region,
			t2.user_group_name, t3.user_role_name
		FROM
			_db_core_users t1
		LEFT JOIN
			_db_core_user_groups t2
		ON
			t1.user_group = t2.user_group_id
		LEFT JOIN
			_db_core_user_roles t3
		ON
			t1.user_role = t3.user_role_id
		WHERE
			(t1.user_name = '%(username)s' OR t1.user_email = '%(username)s')
		AND
			IF('%(password)s' = '*', 1, t1.user_password = '%(password)s')
	";
	protected $singleRow = true;

	/**
	 *
	 * @param <type> $parameters
	 * @return <type>
	 */
	public function checkParameters(&$parameters = null)
	{
		list($username, $password) = $parameters;
		//$parameters = $this->user->prevent_sql_injection(array($username, $username, $password));
		$parameters = $this->protectArray(array('username' => $username, 'password' => $password));
		return true; // the parameters are considered as valid
	}

	/**
	 *
	 * @param boolean $results
	 * @return <type>
	 */
	public function checkResults(&$results = null)
	{
		if (empty($results))
				$results = false; //  to be consistent with the original API
return true;
	}
}

/**
 * Security - Selects users cookie data.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class USER_selectCookieQuery extends PHPDS_query
{
	protected $sql = "
		SELECT
			user_id, cookie_id, pass_crypt
		FROM
			_db_core_session
		WHERE
			id_crypt = '%s'
	";
}

/**
 * Security - Deletes users cookie data.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class USER_deleteCookieQuery extends PHPDS_query
{
	protected $sql = "
		DELETE FROM
			_db_core_session
		WHERE
			cookie_id = %u
	";
}

/**
 * Security - Selects users persistant login details.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org, Ross Kuyper
 *
 */
class USER_selectUserPersistentQuery extends PHPDS_query
{
	protected $sql = " SELECT
			t1.user_id, t1.user_display_name, t1.user_password, t1.user_name, t1.user_email, t1.user_group, t1.user_role, t1.language, t1.timezone as user_timezone, t1.region,
			t2.user_group_name, t3.user_role_name
		FROM
			_db_core_users t1
		LEFT JOIN
			_db_core_user_groups t2
		ON
			t1.user_group = t2.user_group_id
		LEFT JOIN
			_db_core_user_roles t3
		ON
			t1.user_role = t3.user_role_id
		WHERE
			t1.user_id = '%s'
	";
	protected $singleRow = true;

	/**
	 * Override.
	 * @param array $parameters
	 * @return mixed
	 */
	public function invoke($parameters = null)
	{
		$cookie = $parameters[0];

		$id_crypt = substr($cookie, 0, 6);
		$pass_crypt = substr($cookie, 6, 32);

		$found = false;

		// Fetch all instances which match the id crypt
		$persistent_array = $this->db->invokeQuery('USER_selectCookieQuery', $id_crypt);

		// Loop and check for matching pass crypt
		if (!empty($persistent_array)) {
			$persistent_item = end($persistent_array);
			if ($pass_crypt == $persistent_item['pass_crypt']) {
				$cookie_id = $persistent_item['cookie_id'];
				$user_id = $persistent_item['user_id'];
				$found = true;
			}
		}

		// Return false or get the data
		if (!empty($found)) {
			// Delete the old cookie
			$persistent_array = $this->db->invokeQuery('USER_deleteCookieQuery', $cookie_id);

			// Set a new persistent cookie for the next login
			$this->user->setPersistentCookie($user_id);

			$select_user_array = parent::invoke($user_id);

			$this->user->doLogin($select_user_array, "Persistent Login");
		} else {
			return false;
		}
	}
}

/**
 * Security - Insert persistant cookie data.
 * @author Ross Kuyper
 *
 */
class USER_setPersistentCookieQuery extends PHPDS_query
{
	protected $sql = "
		INSERT INTO
			_db_core_session (user_id, id_crypt, pass_crypt, timestamp)
		VALUES
			(%u, '%s', '%s', '%s')
	";

	public function invoke($parameters = null)
	{
		$user_id = $parameters[0];
		$pass_crypt = md5(uniqid(rand(), TRUE));
		$id_crypt = substr(md5(uniqid(rand(), TRUE)), 6, 6);
		$timestamp = time();

		parent::invoke(array($user_id, $id_crypt, $pass_crypt, $timestamp));

		return setcookie('pdspc', $id_crypt . $pass_crypt, $timestamp + 63113852);
	}
}

/**
 * Security - Remove persistant cookie from database
 * @author Ross Kuyper
 *
 */
class USER_deletePersistentCookieQuery extends PHPDS_query
{
	protected $sql = "
		DELETE FROM
			_db_core_session
		WHERE
			(id_crypt = '%s' AND user_id = '%s')
	";

	public function invoke($parameters = null)
	{
		if (!empty($_COOKIE['pdspc'])) {
			$user_id = $parameters[0];

			$id_crypt = substr($_COOKIE['pdspc'], 0, 6);

			parent::invoke(array($id_crypt, $user_id));

			return setcookie('pdspc', 'false', 0);
		}
	}
}

/**
 * Security - The guests group name.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class USER_guestGroupNameQuery extends PHPDS_query
{
	protected $sql = "
			SELECT
				user_group_name
			FROM
				_db_core_user_groups
			WHERE
				user_group_id = %u
		";
	protected $singleValue = true;
	protected $focus = 'user_group_name';

	public function checkParameters(&$parameters = null)
	{
		$settings_array = $this->db->essentialSettings;
		$parameters = $settings_array['guest_group'];
		return true; // the parameters are considered as valid
	}
}

/**
 * Security - The guest roles name.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class USER_guestRoleNameQuery extends PHPDS_query
{
	protected $sql = "
			SELECT
				user_role_name
			FROM
				_db_core_user_roles
			WHERE
				user_role_id = %u
		";
	protected $singleValue = true;
	protected $focus = 'user_role_name';

	public function checkParameters(&$parameters = null)
	{
		$settings_array = $this->db->essentialSettings;
		$parameters = $settings_array['guest_role'];
		return true; // the parameters are considered as valid
	}
}

/**
 * Security - Queries if user is a root user.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class USER_isRootQuery extends PHPDS_query
{
	protected $sql = "
			SELECT
				user_role
			FROM
				_db_core_users
			WHERE
				user_id = %u
		";
	protected $singleValue = true;
	protected $focus = 'user_role';

	public function checkParameters(&$parameters = null)
	{
		list($user_id) = $parameters;
		return intval($user_id) > 0;
	}
}

/**
 * Security - Count Rows.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class USER_isSameGroupQuery extends PHPDS_query
{
	protected $sql = "
		SELECT
			user_id
		FROM
			_db_core_users
		WHERE
			user_id = %u
		%s
		";
	protected $singleValue = true;
	protected $focus = 'user_id';

	public function checkParameters(&$parameters = null)
	{
		list($user_id) = $parameters;
		if (intval($user_id) > 0) {
			$parameters = array($user_id, $this->user->setGroupQuery("AND user_group IN ({$this->user->getGroups()})"));
		} else return false;
	}
}

/**
 * USER - Get extra users roles.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class USER_getExtraRolesQuery extends PHPDS_query
{
	protected $sql = "
		SELECT
			user_role_id
		FROM
			_db_core_user_extra_roles
		WHERE
			user_id = %u
	";
}

/**
 * USER - Get users roles.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class USER_getRolesQuery extends PHPDS_query
{
	protected $sql = "
		SELECT
			user_role
		FROM
			_db_core_users
		WHERE
			user_id = %u
	";

	protected $singleValue = true;

	/**
	 * Initiate query invoke command.
	 *
	 * Parameters are ($user_id, $return_array) - $user_id can be 0 for current user
	 *
	 * @version 1.0.2
	 *
	 * @date 20110215 (greg) (v1.0.1) added checks to avoid error if $select_user_roles_db is not an array
	 * @date 20110315 (jason) (v1.0.2) improved functionality and stability.
	 *
	 * @param array
	 * @return array
	 */
	public function invoke($parameters = null)
	{
		list($user_id, $return_array) = $parameters;
		$configuration = $this->configuration;

		// First try to assign default user id.
		if (empty($user_id)) {
			(! empty($configuration['user_id'])) ? $user_id = $configuration['user_id'] : $user_id = 0;
		}
		// Check if user is a guest.
		if (!empty($user_id)) {
			// Check if we have a saved variable.
			if (empty($this->user->mergeRoles)) {
				// Pull user groups.
				if ($user_id == $configuration['user_id']) {
					$role_main = $configuration['user_role'];
				} else {
					$role_main = parent::invoke($user_id);
				}

				// Get all extra roles for user..
				$role_array = $this->db->invokeQuery('USER_getExtraRolesQuery', $user_id);

				// Set.
				$role_string = '';

				// Check if anything is selected.
				if (is_array($role_array)) {
					// Loop Roles and return string.
					foreach ($role_array as $role_arr) {
						$role_string .= "{$role_arr['user_role_id']},";
					}
				}

				// Merger primary and extra roles.
				$this->user->mergeRoles = rtrim("$role_main," . $role_string, ',');
			}

			// What should we return, array or , string.
			if ($return_array == false) {
				// Ok return string.
				return $this->user->mergeRoles;
			} else {
				// Ok return array.
				return explode(',', $this->user->mergeRoles);
			}
		} else {
			$settings = $this->db->essentialSettings;
			$guest_role = $settings['guest_role'];
			if (empty($guest_role)) throw new PHPDS_Exception('Unable to get the GUEST ROLE from the essential settings');
			return $return_array ? array($guest_role) : $guest_role;
		}
	}
}

/**
 * USER - Get extra users groups.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class USER_getExtraGroupsQuery extends PHPDS_query
{
	protected $sql = "
		SELECT
			user_group_id
		FROM
			_db_core_user_extra_groups
		%s
	";
}

/**
 * USER - Get users groups.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * @date 20110315 (jason) (v1.0.2) improved functionality and stability.
 *
 */
class USER_getGroupsQuery extends PHPDS_query
{
	protected $sql = "
		SELECT
			user_group
		FROM
			_db_core_users
		%s
	";

	protected $singleValue = true;

	/**
	 * Initiate query invoke command.
	 * @param array
	 * @return array
	 */
	public function invoke($parameters = null)
	{
		list($user_id, $return_array) = $parameters;
		$configuration = $this->configuration;

		// First try to assign default user id.
		if (empty($user_id)) {
			(!empty($configuration['user_id'])) ? $user_id = $configuration['user_id'] : $user_id = false;
		}
		// Do we have a root user.
		if ($this->user->isRoot($user_id)) {
			// get all groups.
			$query = '';
		} else {
			$query = " WHERE user_id = $user_id ";
		}
		// Check if user is a guest.
		if (!empty($user_id)) {
			// Check if we have a saved variable.
			if (empty($this->user->mergeGroups)) {
				// Pull user groups.
				if ($user_id == $configuration['user_id']) {
					$group_main = $configuration['user_group'];
				} else {
					$group_main = parent::invoke($query);
				}

				// Get all extra groups for user.
				$group_array = $this->db->invokeQuery('USER_getExtraGroupsQuery', $query);
				// Set.
				$group_string = '';

				// Check if anything is selected.
				if (is_array($group_array)) {
					
					// Loop Groups and return string.
					foreach ($group_array as $group_arr) {
						$group_string .= "{$group_arr['user_group_id']},";
					}
				}

				// Merger primary and extra groups.
				$this->user->mergeGroups = rtrim("$group_main," . $group_string, ',');
			}
	
			// What should we return, array or , string.
			if ($return_array == false) {
				// Ok return string.
				return $this->user->mergeGroups;
			} else {
				// Ok return array.
				return explode(',', $this->user->mergeGroups);
			}
		} else {
			$settings = $this->db->essentialSettings;
			return $settings['guest_group'];
		}
	}
}

/**
 * USER - Get users groups children.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class USER_getGroupsChildrenQuery extends PHPDS_query
{
	protected $sql = "
		SELECT
			user_group_id, parent_group_id, alias
		FROM
			_db_core_user_groups
	";
}

/**
 * USER - Get user roles.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class USER_roleExistQuery extends PHPDS_query
{
	protected $sql = "
		SELECT
			user_role_id
		FROM
			_db_core_user_roles
	";

	/**
	 * Initiate query invoke command.
	 * @param array
	 * @return array
	 */
	public function invoke($parameters = null)
	{
		$role_id = $parameters[0];
		if (empty($this->user->rolesArray)) {
			// Do roles query.
			$roles = parent::invoke();
			// Write array.
			foreach ($roles as $results_array) {
				$this->user->rolesArray[$results_array['user_role_id']] = true;
			}
		}
		// Do we have a role like this?
		if (!empty($this->user->rolesArray[$role_id])) {
			return true;
		} else {
			return false;
		}
	}
}

/**
 * USER - Get user groups.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class USER_groupExistQuery extends PHPDS_query
{
	protected $sql = "
		SELECT
			user_group_id
		FROM
			_db_core_user_groups
	";

	/**
	 * Initiate query invoke command.
	 * @param array
	 * @return array
	 */
	public function invoke($parameters = null)
	{
		$group_id = $parameters[0];
		if (empty($this->user->groupsArray)) {
			// Do groups query.
			$groups = parent::invoke();
			// Write array.
			foreach ($groups as $results_array) {
				$this->user->groupsArray[$results_array['user_group_id']] = true;
			}
		}
		// Do we have a group like this?
		if (!empty($this->user->groupsArray[$group_id])) {
			return true;
		} else {
			return false;
		}
	}
}

/**
 * USER - Check if user belongs to role.
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 */
class USER_belongsToRoleQuery extends PHPDS_query
{
	protected $sql = "
		SELECT
			t1.user_id
		FROM
			_db_core_users t1
		LEFT JOIN
			_db_core_user_extra_roles t2
		ON
			t1.user_id = t2.user_id
		WHERE
			(t1.user_role = %u or t2.user_role_id = %u)
		AND
			(t1.user_id = %u or t2.user_id = %u)
	";

	protected $singleRow = true;

	/**
	 * Initiate query invoke command.
	 * @param array
	 * @return array
	 */
	public function invoke($parameters = null)
	{
		list($user_id, $user_role) = $parameters;
		// We need to check what user role this user belongs to.
		// This user belongs to root role, so it is safe to allow.
		if ($this->user->isRoot($user_id)) return true;
		// Check if we have a user id.
		if (empty($user_id)) {
			// Use default id.
			(!empty($this->configuration['user_id'])) ? $user_id = $this->configuration['user_id'] : $user_id = false;
		}
		// First lets query if this user belongs to given role.
		$check_user_in_role_db = parent::invoke(array($user_role, $user_role, $user_id, $user_id));

		// Lets check if this user exists.
		if ($check_user_in_role_db['user_id'] == $user_id) {
			return true;
		} else {
			return false;
		}
	}
}

