<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */

/**
 * Controller Class: Handles system cron job admin.
 * @author Jason Schoeman
 * @return string
 */
class cronjobAdmin extends PHPDS_controller
{

	/**
	 * Execute Controller
	 * @author Jason Schoeman
	 */
	public function execute()
	{
		$this->template->heading(_('Edit Cronjob'));
		$this->template->info(_('Edit cronjobs to run on requested intervals or on a specific time. a Cronjob executes a script to run without human intervention, this is mostly used to run maintenance scripts.'));

		// Define.
		$cron_type_selected_0 = false;
		$cron_type_selected_1 = false;
		$cron_type_selected_2 = false;
		$log_selected_1 = false;
		$log_selected_2 = false;
		// Determine menu id.
		if (!empty($this->security->get['ec'])) {
			$d_menu_id = $this->security->get['ec'];
		} else if (!empty($this->security->post['menu_id'])) {
			$d_menu_id = $this->security->post['menu_id'];
		}

		// Check if menu item exists and of what type.
		$edit = $this->db->InvokeQuery('PHPDS_selectEditMenuQuery', $d_menu_id);

		// Locate menu name.
		$menu_name = $this->navigation->determineMenuName($edit['menu_name'], $edit['menu_link'], $edit['menu_id']);

		// Check if we need to edit an existing cron or cron.
		if (!empty($edit['menu_id'])) {

			// On Save
			if (!empty($this->security->post['save'])) {
				// Get saved values.
				$edit = $this->security->post;
				$edit['second'] = 0;

				// Error checking.
				if (empty($edit['menu_id']) || !isset($this->security->post['cron_type']) || !isset($this->security->post['log_cron'])) {
					$error[1] = true;
					$this->template->warning(_('You did not complete all the required fields.'));
				}
				if ($edit['cron_type'] == 1) {
					if (empty($edit['year']) || empty($edit['month']) || empty($edit['day']) || !isset($this->security->post['hour']) || !isset($this->security->post['minute'])) {
						$error[2] = true;
						$this->template->warning(_('You must provide a date on selected run type.'));
					}
					$cron_time = mktime($edit['hour'], $edit['minute'], $edit['second'], $edit['month'], $edit['day'], $edit['year']);
					if ($cron_time <= time()) {
						$error[3] = true;
						$this->template->warning(_('Cron cannot be set to run in the past, you will need a time machine for this!'));
					}
				}

				// If all is ok, save cron.
				if (empty($error)) {
					// Run the query
					$this->db->invokeQuery('PHPDS_saveCronQuery',
							$edit['menu_id'],
							$edit['cron_desc'],
							$edit['cron_type'],
							$edit['log_cron'],
							$edit['last_execution'],
							$edit['year'],
							$edit['month'],
							$edit['day'],
							$edit['hour'],
							$edit['minute']);

					// Output success
					$this->template->ok(sprintf(_('Cronjob %s saved.'), $menu_name));
				}
			}
			// last_executed
			if (!empty($edit['last_execution'])) {
				$last_executed = $this->core->formatTimeDate($edit['last_execution']);
			} else {
				$last_executed = $expectancy = _('Never!');
			}
			// cron_type
			switch ($edit['cron_type']) {
				case 0:
					$cron_type_selected_0 = 'checked';
					$expectancy = _('Never!');
					break;
				case 1:
					$cron_type_selected_1 = 'checked';
					// Check empty values.
					if (empty($edit['hour'])) $edit['hour'] = 0;
					if (empty($edit['minute'])) $edit['minute'] = 0;
					$edit['second'] = 0;
					// Create confirm date.
					if (empty($error[2])) {
						$expectancy = sprintf(_('Expected to run once on %s.'), $this->core->formatTimeDate(mktime($edit['hour'], $edit['minute'], $edit['second'], $edit['month'], $edit['day'], $edit['year'])));
					}
					break;
				case 2:
					$cron_type_selected_2 = 'checked';
					// Convert to hours.
					$year_in_hr = ($edit['year'] * 8765.81277);
					$month_in_hr = ($edit['month'] * 730.484398);
					$day_in_hr = ($edit['day'] * 24);
					$hr_in_hr = $edit['hour'];
					$minutes_in_hr = ($edit['minute'] / 60);
					// hours
					$hours = round($year_in_hr + $month_in_hr + $day_in_hr + $hr_in_hr + $minutes_in_hr, 2);
					// Create total Months/Days/Hours/Minutes.
					if ($hours >= 730.484398) {
						// months
						$months = round($hours / 730.484398, 2);
						$expectancy = sprintf(_('Will run every %s month(s).'), $months);
					} else if ($hours >= 24) {
						// days
						$days = round($hours / 24, 2);
						$expectancy = sprintf(_('Will run every %s day(s).'), $days);
					} else if ($hours >= 1) {
						$expectancy = sprintf(_('Will run every %s hour(s).'), $hours);
					} else if ($hours < 1) {
						// minutes
						$minutes = ($hours * 60);
						$expectancy = sprintf(_('Will run every %s minute(s).'), $minutes);
					}
					break;
			}
			// log_cron
			($edit['log_cron'] == 1) ? $log_selected_1 = 'checked' : $log_selected_2 = 'checked';

			// Load views.
			$view = $this->factory('views');

			// Set Array.
			$view->set('e', $edit);

			// Set Values.
			$view->set('self_url', $this->navigation->selfUrl());
			$view->set('menu_name', $menu_name);
			$view->set('cron_type_selected_0', $cron_type_selected_0);
			$view->set('cron_type_selected_1', $cron_type_selected_1);
			$view->set('cron_type_selected_2', $cron_type_selected_2);
			$view->set('last_executed', $last_executed);
			$view->set('expectancy', $expectancy);
			$view->set('log_selected_1', $log_selected_1);
			$view->set('log_selected_2', $log_selected_2);

			// Output Template.
			$view->show();
		} else {
			// Require database array.
			$this->template->warning('The cronjob you wish to edit does not exist or it was deleted from the menu system!');
		}
	}
}

return 'cronjobAdmin';