<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */

/**
 * Controller Class: List groups.
 * @author Jason Schoeman
 * @return string
 */
class UserGroupAdminList extends PHPDS_controller
{

	/**
	 * Execute Controller
	 * @author Jason Schoeman
	 */
	public function execute()
	{
		// Header information
		$this->template->heading(_('User Group Management'));
		$this->template->info(_('Listing User Groups that allows the system to restrict user access to only access data belonging to the same group. Allowing the creation of data accessing policies. These settings does not restrict script accessing. If you assign a user to a parent group, user will be able to access all child groups as well.'));

		// Should we delete groups users?
		if (!empty($this->security->get['dgu'])) {
			// Do we have a naughty branch admin that is trying to delete things he should not?
			if (!$this->user->belongsToGroup(false, $this->security->get['dgu']) && $this->configuration['user_group'] != $this->security->get['dgu']) {
				$this->template->warning(_('You cannot delete this groups users! You do not have permission or you are in this group.'));
				$error_[0] = true;
			}
			// We should be safe to delete now.
			if (empty($error_)) {
				if ($this->db->deleteQuick('_db_core_users', 'user_group', $this->security->get['dgu'])) {
					// Delete old user groups values.
					$this->db->deleteQuick('_db_core_user_extra_groups', 'user_group_id', $this->security->get['dgu']);
					$this->template->ok(sprintf(_('All user from group %s was deleted.'), $this->security->get['dgu']));
				} else {
					$this->template->warning(sprintf(_('No users for group "%s" to delete.'), $this->security->get['dgu']));
				}
			}
		}
		// Should we delete group?
		if (!empty($this->security->get['dg'])) {
			// Do we have a naughty branch admin that is trying to delete things he should not?
			if (!$this->user->belongsToGroup(false, $this->security->get['dg']) && $this->configuration['user_group'] != $this->security->get['dg']) {
				$this->template->warning(_('You cannot delete this group! You do not have permission or the group is your own.'));
				$error[0] = true;
			}
			// We should be safe to delete now.
			if (empty($error)) {
				// Lets delete the item.
				$group_deleted = $this->db->deleteQuick('_db_core_user_groups', 'user_group_id', $this->security->get['dg'], 'user_group_name');
				// We also need to delete the extra groups from the list.
				$this->db->deleteQuick('_db_core_user_extra_groups', 'user_group_id', $this->security->get['dg']);
				// We now need to set the user database to also delete this group from user if he belongs to it.
				$this->db->invokeQuery('PHPDS_updateDeletedGroupUsersQuery', $this->security->get['dg']);

				if ($group_deleted) {
					$this->template->ok(sprintf(_('Group %s was deleted.'), $group_deleted));
				} else {
					$this->template->warning(sprintf(_('No group "%s" to delete.'), $this->security->get['dg']));
				}
			}
		}
		$RESULTS = $this->db->invokeQuery('PHPDS_readGroupQuery');

		// Load views.
		$view = $this->factory('views');

		// Set Array.
		$view->set('RESULTS', $RESULTS);

		// Output Template.
		$view->show();
	}
}

return 'UserGroupAdminList';