<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */

/**
 * Controller Class: Recover lost passwords.
 * @author Jason Schoeman
 * @return string
 */
class LostPassword extends PHPDS_controller
{

	/**
	 * Execute Controller
	 * @author Jason Schoeman
	 */
	public function execute()
	{
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Load Extra Classes ////////////////////////////////////////////////////////////////////////////////////////////////////
		$email = $this->factory('mailer'); ///////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// Load page.
		$reset_page_url = $this->navigation->buildURL('4283172353', 'eun=');

		// Set.
		$username = false;
		$this->template->heading(_('Recover Lost Password'));
		$this->template->info(_('If you have forgotten your password, you may use this form to recover it. Simply type in your username or email address, your new password will then be emailed to you after you have submitted your details.'));
		// Sending password instructions.
		if (isset($this->security->post['send'])) {

			// Define.
			(empty($_SESSION['hold'])) ? $hold = false : $hold = $_SESSION['hold'];
			// If user tries to reset another user in less then 60 seconds, reply with an error.
			$seconds_ = $this->configuration['time'] - $hold;
			if ($seconds_ < 60) {
				$this->template->warning(sprintf(_('You can not reset another user within 60 seconds, you have waited %s seconds.'), $seconds_), false, false);
				$error[1] = true;
			}
			// Check for missing fields.
			if (empty($this->security->post['user_name'])) {
				$this->template->warning(_('You have missed some fields, please try again!'), false, false);
				$error[2] = true;
			}

			if (empty($error)) {

				// Get user data.
				$username = $this->security->post['user_name'];
				// Do query getting user.
				$user_array = $this->db->invokeQuery('PHPDS_ReadUserQuery', $username, $username);
				$edit = $user_array;
				if (($edit['user_name'] === $this->security->post['user_name']) || ($edit['user_email'] == $this->security->post['user_name'])) {
					// Create url where password needs to be reset.
					$reset_page_url = $reset_page_url . md5($edit['user_name'] . $edit['user_email'] . $edit['user_password']);
					
					$to = $edit['user_email'];
					$subject = sprintf(_('%s password recovery.'), $this->configuration['scripts_name_version']);
					$message = sprintf(_("Dear %s member,\r\n\r\nYou requested for your password to be reset,\r\nplease click on link below to enter a new password\r\n\r\n%s\r\n\r\nUse your new password to log in after it has been changed.\r\n\r\nThank you,\r\n%s"), $this->configuration['scripts_name_version'], $reset_page_url, $this->configuration['absolute_url'], $this->configuration['absolute_url']);

					// Send new password email.
					if ($email->sendmail("$to", $subject, $message)) {
						$this->template->ok(_('Please check your email inbox for further instructions on changing your password.'));
						$_SESSION['hold'] = $this->configuration['time'];
					} else {
						$this->template->warning(_('An error occurred while trying to send this email. Please notify the Administrator of this problem.'));
					}
				} else {
					$this->template->warning(_('The specified user was not found, make sure your username or email address are typed in correctly. Note that they are case sensitive. It is also possible that you may have been removed from the system.'));
				}
			}
		}

		// Load views.
		$view = $this->factory('views');

		// Set Buttons.
		$view->set('send_recovery_instructions', _('Send Recovery Instructions'));

		// Set Values.
		$view->set('self_url', $this->navigation->selfUrl());
		$view->set('username', $username);
		$view->set('post_validation', $this->security->postValidation());

		// Output Template.
		$view->show();
	}
}

return 'LostPassword';
